require "defines"

isTurretWagonEntity = function(entity)
	return (isValid(entity) and entity.name == "turret-wagon")	
end

entityBuilt = function(event)
	--debugLog("On build")
	local entity = event.created_entity
	if isTurretWagonEntity(entity) then
		debugLog("entity is turretwagon")
		local wagon = {entity = entity}
		wagon.proxy=createProxyTurretWagon(entity.position,entity.surface)
		wagon.proxy.destructible = false
		global.turretwagons = valueOrNewTable(global.turretwagons)
		table.insert(global.turretwagons, wagon)
		--debugLog("new wagon: " .. #global.turretwagons)
	end
end

onTickMain = function(event) -- Move each turret to follow its wagon
	if global.turretwagons ~= nil then
		for i,wagon in ipairs(global.turretwagons) do
			if isValid(wagon.proxy) then
				wagon.proxy.teleport({x=wagon.entity.position.x,y=wagon.entity.position.y})
			end
		end
	end
end

script.on_event(defines.events.on_tick, onTickMain)

function createProxyTurretWagon(position, surface)
	local proxy = surface.create_entity{name="turret-wagon-proxy", position=position, force=game.players[1].force}
	return proxy
end

script.on_event(defines.events.on_built_entity, entityBuilt)
script.on_event(defines.events.on_robot_built_entity, entityBuilt)

entityRemoved = function (event)
	if isTurretWagonEntity(event.entity) then
		local newFunction = function (val) return val.entity == event.entity end
		local wagon = getWagonFromEntity(global.turretwagons, event.entity)
		if isWagonValid(wagon) then
			if isValid(wagon.proxy) then
				wagon.proxy.destroy()
				wagon.proxy = nil
			end
			global.turretwagons = nilIfEmptyTable(remove_if(newFunction, global.turretwagons))
		end
	end
end


script.on_event(defines.events.on_preplayer_mined_item, entityRemoved)
script.on_event(defines.events.on_robot_pre_mined, entityRemoved)
script.on_event(defines.events.on_entity_died, entityRemoved)


function getWagonFromEntity(wagons, entity)
	if wagons == nil then return nil end
	--debugLog("tankers not nil")
	for i,value in ipairs(wagons) do
		if isValid(value.entity) and entity == value.entity then
			return value
		end
	end
end

function isValid(entity)
	return(entity ~= nil and entity.valid)
end

isWagonValid = function(wagon)
	return(wagon ~= nil and (isValid(wagon.proxy)))
end

function debugLog(message)
	if false then -- set for debug
		for i,player in ipairs(game.players) do
			player.print(message)
		end
	end
end 


function valueOrNewTable(value)
	if value == nil then
		--debugLog("NewTable")
		return {}
	else
		return value
	end
end

function nilIfEmptyTable(value)
	if value == nil or #value < 1 then
		return nil
	else 
		return value
	end
end

function remove_if(func, arr)
  if arr == nil then return nil end
  local new_array = {}
  for _,v in ipairs(arr) do
    if not func(v) then table.insert(new_array, v) end
  end
  return new_array
end